package com.kz.grbl;

import java.util.List;

import org.jzy3d.colors.Color;
import org.jzy3d.maths.Coord2d;
import org.jzy3d.maths.Coord3d;
import org.jzy3d.plot3d.primitives.AbstractComposite;
import org.jzy3d.plot3d.primitives.AbstractDrawable;
import org.jzy3d.plot3d.primitives.Point;
import org.jzy3d.plot3d.primitives.Polygon;
import org.jzy3d.plot3d.primitives.Quad;
import org.jzy3d.plot3d.rendering.scene.Graph;

public class Cylinder extends AbstractComposite {
	Graph graph;
	Coord3d position; 

	public Cylinder(Graph g) {
		graph = g;
	}

	public void setData(Coord3d position, float height, float radius, int slices, int rings, Color color) {
		// Create sides
		top = new Polygon();
		low = new Polygon();
		this.position  = position;

		for (int i = 0; i < slices; i++) {
			float angleBorder1 = (float) i * 2 * (float) Math.PI / (float) slices;
			float angleBorder2 = (float) (i + 1) * 2 * (float) Math.PI / (float) slices;

			Coord2d border1 = new Coord2d(angleBorder1, radius).cartesian();
			Coord2d border2 = new Coord2d(angleBorder2, radius).cartesian();

			Quad face = new Quad();
			face.add(new Point(new Coord3d(position.x + border1.x, position.y + border1.y, position.z)));
			face.add(new Point(new Coord3d(position.x + border1.x, position.y + border1.y, position.z + height)));
			face.add(new Point(new Coord3d(position.x + border2.x, position.y + border2.y, position.z + height)));
			face.add(new Point(new Coord3d(position.x + border2.x, position.y + border2.y, position.z)));
			face.setColor(color);
			face.setWireframeDisplayed(false);

			// add the polygon to the cylinder
			add(face);

			// compute top and low faces
			low.add(new Point(new Coord3d(position.x + border1.x, position.y + border1.y, position.z)));
			top.add(new Point(new Coord3d(position.x + border1.x, position.y + border1.y, position.z + height)));
		}
		low.setColor(color);
		top.setColor(color);
		add(top);
		add(low);
		
	}

	public void clean() {
		List<AbstractDrawable> drawables = this.getDrawables();
//		for (int i = 0; i < drawables.size(); i++) {
//			AbstractDrawable drawable = drawables.get(i);
//			this.remove(drawable);
//			graph.remove(drawable);
//		}
		graph.remove(this, false);
	}

	private Polygon top;
	private Polygon low;
}
