package com.kz.grbl;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.nio.charset.Charset;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jzy3d.analysis.AbstractAnalysis;
import org.jzy3d.analysis.AnalysisLauncher;
import org.jzy3d.chart.Chart;
import org.jzy3d.chart.controllers.mouse.picking.AWTMousePickingPan2dController;
import org.jzy3d.chart.controllers.mouse.picking.NewtMousePickingPan2dController;
import org.jzy3d.chart.factories.AWTChartComponentFactory;
import org.jzy3d.colors.Color;
import org.jzy3d.maths.Coord3d;
import org.jzy3d.picking.IObjectPickedListener;
import org.jzy3d.picking.PickingSupport;
import org.jzy3d.plot3d.primitives.AbstractDrawable;
import org.jzy3d.plot3d.primitives.LineStrip;
import org.jzy3d.plot3d.primitives.Point;
import org.jzy3d.plot3d.primitives.pickable.PickablePoint;
import org.jzy3d.plot3d.rendering.canvas.CanvasAWT;
import org.jzy3d.plot3d.rendering.canvas.Quality;
import org.jzy3d.plot3d.transform.Rotate;
import org.jzy3d.plot3d.transform.Transform;





public class SurfaceDemo extends AbstractAnalysis {
	Utils utils;
	Cylinder cylinder;
	MyPickablePoint lastClickedPoint;
	String angleTxt = "";

	public static void main(String[] args) throws Exception {
		AnalysisLauncher.open(new SurfaceDemo());
	}

	@Override
	public void init() {
		try {
			
			System.out.println("Creating chart! Thread: " + Thread.currentThread().getName());
			// Create a chart
			chart = AWTChartComponentFactory.chart(Quality.Advanced, getCanvasType());
			chart.getView().setMaximized(true);
			CanvasAWT canvas = (CanvasAWT) chart.getCanvas();
			canvas.getAnimator().setUpdateFPSFrames(500, System.out);

			canvas.getAnimator().start();
			System.out.println("Animator started?: " + canvas.getAnimator().isStarted());

			String filePath = null;
			if (OSValidator.isUnix())
				filePath = "/home/klemen/googledrive/TerasaBoris/grbl";
			else
				filePath = "c:\\Users\\klemen\\Google Drive\\TerasaBoris\\grbl";

			Path path = FileSystems.getDefault().getPath(filePath, "data.csv");
			List<String> lines;
			lines = Files.readAllLines(path, Charset.forName("UTF-8"));
			utils = new Utils();

			ArrayList<String> line = new ArrayList<String>();

			// Point previousPolyPoint = null;
			LineStrip ls_p = null;
			int previousSurfaceNo = -1;
			int previousEdgeNo = -1;
			int pointId = -1;
			for (String string : lines) {
				line.add(string);
				String splitted[] = string.split(";");
				int surfaceNo = Integer.valueOf(splitted[0].split("_")[1]);
				int edgeNo = Integer.valueOf(splitted[0].split("_")[2]);
				int colorNo = surfaceNo % Color.COLORS.length;

				if (splitted[0].startsWith("EDGE")) {

					Float x1 = Float.valueOf(splitted[1].replace(",", "."));
					Float y1 = Float.valueOf(splitted[2].replace(",", "."));
					Float z1 = Float.valueOf(splitted[3].replace(",", "."));
					Coord3d p1 = new Coord3d(x1, y1, z1);
					int id1 = pointId++;
					MyPickablePoint mp1 = utils.createOrFindMyPickablePoint(id1, surfaceNo, edgeNo, p1,
							Color.COLORS[colorNo], 5);

					Float x2 = Float.valueOf(splitted[4].replace(",", "."));
					Float y2 = Float.valueOf(splitted[5].replace(",", "."));
					Float z2 = Float.valueOf(splitted[6].replace(",", "."));
					Coord3d p2 = new Coord3d(x2, y2, z2);
					int id2 = pointId++;
					MyPickablePoint mp2 = utils.createOrFindMyPickablePoint(id2, surfaceNo, edgeNo, p2,
							Color.COLORS[colorNo], 5);

					LineStrip ls = new LineStrip();
					ls.add(mp1);
					ls.add(mp2);
					ls.setWireframeColor(Color.COLORS[colorNo]);
					chart.getScene().getGraph().add(ls);
				} else if (splitted[0].startsWith("POLY")) {
					if (surfaceNo != previousSurfaceNo || edgeNo != previousEdgeNo) {
						if (ls_p != null) {
							ls_p.setWireframeColor(Color.COLORS[colorNo]);
							chart.getScene().getGraph().add(ls_p);
						}
						ls_p = new LineStrip();
					}

					Float x = Float.valueOf(splitted[1].replace(",", "."));
					Float y = Float.valueOf(splitted[2].replace(",", "."));
					Float z = Float.valueOf(splitted[3].replace(",", "."));
					Coord3d coord = new Coord3d(x.floatValue(), y.floatValue(), z.floatValue());

					int id = pointId++;
					MyPickablePoint pickablePoint = utils.createOrFindMyPickablePoint(id, surfaceNo, edgeNo, coord,
							Color.COLORS[colorNo], 5);

					if (ls_p != null) {
						ls_p.add(pickablePoint);
					}
					previousSurfaceNo = surfaceNo;
					previousEdgeNo = edgeNo;
				}
			}

			// Creates the 3d object
			System.out.println("Thread: " + Thread.currentThread().getName());
			chart.getScene().getGraph().add(utils.points);
			chart.getAxeLayout().setXAxeLabel("X");
			chart.getAxeLayout().setYAxeLabel("Y");
			chart.getAxeLayout().setZAxeLabel("Z");

			chart.getView().setSquared(false);

			// camMouse = new SubClassNewtCameraMouseController(chart);
			// camMouse = new NewtCameraMouseController(chart);

			enablePicking(utils.points, chart);
			System.out.println("Thread: " + Thread.currentThread().getName());

			canvas.addKeyListener(new KeyListener() {

				@Override
				public void keyReleased(KeyEvent e) {
					// TODO Auto-generated method stub

				}

				@Override
				public void keyPressed(KeyEvent e) {
					// TODO Auto-generated method stub
					System.out.println(e.getKeyCode());
					if (e.getKeyCode() == 150 || e.getKeyCode() == 152) {
						double angleDelta = 0;
						if (e.getKeyCode() == 150) {
							System.out.println("UP");
							angleDelta = 1;
						}

						if (e.getKeyCode() == 152) {
							System.out.println("DOWN");
							angleDelta = -1;
						}
						Double value = Double.valueOf(angleTxt);
						value = value + angleDelta;
						// angle.setText(value.toString());

						Rotate rotate = new Rotate(angleDelta, new Coord3d(0, 1, 0));
						Transform t = new Transform();
						t.add(rotate);
						Iterator<AbstractDrawable> it = chart.getScene().getGraph().getAll().iterator();
						while (it.hasNext()) {
							AbstractDrawable dr = it.next();
							if (dr.getClass().getName().equals("com.kz.grbl.MyPickablePoint"))
								dr.applyGeometryTransform(t);
							else
								System.out.println(dr.getClass().getName());
						}
					}

					chart.render();

				}

				@Override
				public void keyTyped(KeyEvent e) {
					// TODO Auto-generated method stub
					
				}
			});

		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private void enablePicking(List<MyPickablePoint> points, Chart chart) {
		AWTMousePickingPan2dController<?, ?> mousePicker = new AWTMousePickingPan2dController<>(chart, 5);
		PickingSupport picking = mousePicker.getPickingSupport();

		for (PickablePoint p : points) {
			picking.registerPickableObject(p, p);
		}

		picking.addObjectPickedListener(new IObjectPickedListener() {
			@Override
			public void objectPicked(List<?> picked, PickingSupport ps) {
				if (picked.size() > 0) // && (System.currentTimeMillis() -
										// camMouse.clickTimeMillis < 200)) {
				{
					System.out.println("Size: " + picked.size());
					for (int i = 0; i < picked.size(); i++) {
						if (picked.get(i).getClass().getName().equals("com.kz.grbl.MyPickablePoint")) {
							((Point) picked.get(i)).setBoundingBoxColor(Color.BLACK);
							((Point) picked.get(i)).setBoundingBoxDisplayed(true);
							MyPickablePoint mp = ((MyPickablePoint) picked.get(i));
							System.out.println(mp.toString());
							addCylinder(mp.xyz);
							lastClickedPoint = mp;
							Visualizer visualizer = new Visualizer(utils.points, SurfaceDemo.this.getChart(), lastClickedPoint);
							visualizer.start();
						} else if (picked.get(i).getClass().getName()
								.equals("org.jzy3d.plot3d.primitives.pickable.PickablePolygon")) {
							System.out.println(picked.get(i).getClass().getName());
						} else {
							System.out.println(picked.get(i).getClass().getName());
						}

					}

				}
			}
		});
	}

	public void addCylinder(Coord3d coord3d) {
		System.out.println("Thread: " + Thread.currentThread().getName());

		Color color = Color.RED; // mapper.getColor(new Coord3d(0,0,height));
		color.a = 0.55f;

		if (cylinder != null) {
			cylinder.clean();
		}

		cylinder = new Cylinder(this.chart.getScene().getGraph());
		double height = 0.7;
		cylinder.setData(coord3d, (float) height, (float) 0.1, 15, 5, color);
		cylinder.setWireframeDisplayed(true);
		cylinder.setWireframeColor(Color.BLACK);
		chart.getScene().getGraph().add(cylinder);

	}

	
	
}
